// Copyright 2024 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


import SwiftUI

import ArcGIS

class Model: ObservableObject {

    let graphicsOverlay = GraphicsOverlay()

    let startGraphic: Graphic = {
        let symbol = SimpleMarkerSymbol(style: .circle, color: .white, size: 8)
        symbol.outline = SimpleLineSymbol(style: .solid, color: .black, width: 1)
        let graphic = Graphic(symbol: symbol)
        return graphic
    }()

    let endGraphic: Graphic = {
        let symbol = SimpleMarkerSymbol(style: .circle, color: .black, size: 9)
        let graphic = Graphic(symbol: symbol)
        return graphic
    }()

    let routeGraphic: Graphic = {
        let symbol = SimpleLineSymbol(style: .solid, color: .blue, width: 3)
        let graphic = Graphic(symbol: symbol)
        return graphic
    }()

    init() {
        graphicsOverlay.addGraphics([routeGraphic, startGraphic, endGraphic])
    }

    private let routeTask = RouteTask(
        url: URL(string: "https://route-api.arcgis.com/arcgis/rest/services/World/Route/NAServer/Route_World")!
    )

    var directions: [DirectionManeuver] = []

    func solveRoute(from start: Point, to end: Point) async throws {

        let routeParameters = try await routeTask.makeDefaultParameters()
        routeParameters.returnsDirections = true
        routeParameters.setStops([Stop(point: start), Stop(point: end)])

        let routeResult = try await routeTask.solveRoute(using: routeParameters)
        if let firstRoute = routeResult.routes.first {
            routeGraphic.geometry = firstRoute.geometry
            directions = firstRoute.directionManeuvers
        }

    }

}

struct ContentView: View {

    @StateObject private var model = Model()

    @State private var isShowingDirections = false

    @State private var startPoint: Point?
    @State private var endPoint: Point?

    @State var map = {

        let map = Map(basemapStyle: .arcGISNavigation)
        map.initialViewpoint = Viewpoint(latitude: 34.05293, longitude: -118.24368, scale: 2e5)

        return map
    }()

    var body: some View {

        MapView(map: map, graphicsOverlays: [model.graphicsOverlay])

            .onSingleTapGesture { _, mapPoint in
                if startPoint == nil {
                    startPoint = mapPoint
                    model.startGraphic.geometry = startPoint
                } else {
                    endPoint = mapPoint
                    model.endGraphic.geometry = endPoint
                }
            }

            .task(id: endPoint) {
                guard let startPoint = startPoint, let endPoint = endPoint else { return }
                do {
                    try await model.solveRoute(from: startPoint, to: endPoint)
                } catch {
                    print(error)
                }
            }

            .toolbar {
                ToolbarItemGroup(placement: .bottomBar) {

                    Button("Show directions") {
                        isShowingDirections = true
                    }

                    .popover(isPresented: $isShowingDirections) {
                        NavigationView {

                            List(model.directions, id: \.text) { directionManeuver in
                                Text(directionManeuver.text)
                            }
                            .navigationTitle("Directions")
                            .navigationBarTitleDisplayMode(.inline)
                            .toolbar {
                                ToolbarItem(placement: .confirmationAction) {
                                    Button("Done") {
                                        isShowingDirections = false
                                    }
                                }
                            }

                        }
                        .navigationViewStyle(.stack)
                        .frame(idealWidth: 320, idealHeight: 428)
                    }

                }
            }

    }

}


